#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_skin.interfaces import IInnerPage
from pyams_skin.layer import IPyAMSLayer
from pyams_utils.interfaces import VIEW_SYSTEM_PERMISSION
from pyams_workflow.interfaces import IWorkflowVersion, IWorkflowState, IWorkflow
from pyams_zmi.interfaces import IAdminView
from pyams_zmi.interfaces.menu import IContentManagementMenu
from z3c.table.interfaces import IValues, IColumn

# import packages
from pyams_pagelet.pagelet import pagelet_config
from pyams_security.principal import MissingPrincipal
from pyams_security.utility import get_principal
from pyams_skin.container import ContainerView
from pyams_skin.table import BaseTable, I18nColumn
from pyams_skin.viewlet.menu import MenuItem
from pyams_utils.adapter import adapter_config, ContextRequestViewAdapter
from pyams_utils.date import format_datetime, SH_DATETIME_FORMAT
from pyams_utils.text import text_to_html
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.layer import IAdminLayer
from z3c.table.column import GetAttrColumn
from zope.interface import implementer, Interface

from pyams_workflow import _


@viewlet_config(name='workflow-history.menu', context=IWorkflowVersion, layer=IAdminLayer,
                manager=IContentManagementMenu, permission=VIEW_SYSTEM_PERMISSION, weight=999)
class WorkflowVersionHistoryMenuItem(MenuItem):
    """Workflow history menu item"""

    label = _("Version history")
    icon_class = 'fa-history'
    url = '#version-history.html'


class WorkflowVersionHistoryTable(BaseTable):
    """Workflow version history table"""

    @property
    def title(self):
        translate = self.request.localizer.translate
        return translate(_("Version {version} history")).format(version=IWorkflowState(self.context).version_id)

    cssClasses = {'table': 'table table-bordered table-striped table-hover table-tight datatable'}
    sortOn = None
    hide_body_toolbar = True

    @property
    def data_attributes(self):
        return {
            'table': {
                'data-ams-datatable-global-filter': 'false',
                'data-ams-datatable-info': 'false',
                'data-ams-datatable-sort': 'false',
                'data-ams-datatable-sdom': "t<'dt-row dt-bottom-row'<'text-right'p>>",
                'data-ams-datatable-display-length': '20',
                'data-ams-datatable-pagination-type': 'bootstrap_prevnext'
            }
        }


@adapter_config(context=(IWorkflowVersion, IPyAMSLayer, WorkflowVersionHistoryTable), provides=IValues)
class WorkflowVersionHistoryValuesAdapter(ContextRequestViewAdapter):
    """Workflow version history values adapter"""

    @property
    def values(self):
        return IWorkflowState(self.context).history


@adapter_config(name='date', context=(Interface, IPyAMSLayer, WorkflowVersionHistoryTable), provides=IColumn)
class WorkflowVersionHistoryDateColumn(I18nColumn, GetAttrColumn):
    """Workflow version history date column"""

    _header = _("Date")
    attrName = 'date'
    cssClasses = {'td': 'nowrap'}
    weight = 1

    def renderCell(self, item):
        return format_datetime(item.date, SH_DATETIME_FORMAT, request=self.request)


@adapter_config(name='state', context=(Interface, IPyAMSLayer, WorkflowVersionHistoryTable), provides=IColumn)
class WorkflowVersionHistoryTargetColumn(I18nColumn, GetAttrColumn):
    """Workflow version history source column"""

    _header = _("New state")
    attrName = 'target_state'
    weight = 5

    def getValue(self, obj):
        state = super(WorkflowVersionHistoryTargetColumn, self).getValue(obj)
        workflow = IWorkflow(self.context, None)
        if workflow is None:
            return state
        try:
            term = workflow.states.getTerm(state)
        except LookupError:
            return state or '--'
        else:
            translate = self.request.localizer.translate
            return translate(term.title)


@adapter_config(name='name', context=(Interface, IPyAMSLayer, WorkflowVersionHistoryTable), provides=IColumn)
class WorkflowVersionHistoryPrincipalColumn(I18nColumn, GetAttrColumn):
    """Workflow version history principal column"""

    _header = _("Modifier")
    attrName = 'principal'
    weight = 10

    def renderCell(self, item):
        principal = get_principal(self.request, item.principal)
        if isinstance(principal, MissingPrincipal):
            return '--'
        else:
            return principal.title


@adapter_config(name='comment', context=(Interface, IPyAMSLayer, WorkflowVersionHistoryTable), provides=IColumn)
class WorkflowVersionHistoryCommentColumn(I18nColumn, GetAttrColumn):
    """Workflow version history comment column"""

    _header = _("Comment")
    weight = 15

    def renderCell(self, item):
        return text_to_html(item.comment or '')


@pagelet_config(name='version-history.html', context=IWorkflowVersion, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
@implementer(IInnerPage, IAdminView)
class WorkflowVersionHistoryView(ContainerView):
    """Workflow version history view"""

    table_class = WorkflowVersionHistoryTable
